@@ -10,7 +10,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import builtins
 import datetime
 from http import client as http_client
 import io
@@ -21,7 +20,6 @@ from unittest import mock
 from oslo_config import cfg
 from oslo_utils import uuidutils
 import requests
-import sendfile
 
 from ironic.common import exception
 from ironic.common.glance_service import image_service as glance_v2_service
@@ -195,6 +193,34 @@ class HttpImageServiceTestCase(base.TestCase):
         head_mock.assert_called_once_with(self.href, verify=False,
                                           timeout=60)
 
+    @mock.patch.object(requests, 'head', autospec=True)
+    def test_validate_href_path_forbidden(self, head_mock):
+        cfg.CONF.set_override('webserver_verify_ca', 'True')
+
+        response = head_mock.return_value
+        response.status_code = http_client.FORBIDDEN
+        url = self.href + '/'
+        resp = self.service.validate_href(url)
+        head_mock.assert_called_once_with(url, verify=True,
+                                          timeout=60)
+        self.assertEqual(http_client.FORBIDDEN, resp.status_code)
+
+    @mock.patch.object(requests, 'head', autospec=True)
+    def test_validate_href_path_redirected(self, head_mock):
+        cfg.CONF.set_override('webserver_verify_ca', 'True')
+
+        response = head_mock.return_value
+        response.status_code = http_client.MOVED_PERMANENTLY
+        url = self.href + '/'
+        new_url = 'http://new-url'
+        response.headers = {'location': new_url}
+        exc = self.assertRaises(exception.ImageRefIsARedirect,
+                                self.service.validate_href,
+                                url)
+        self.assertEqual(new_url, exc.redirect_url)
+        head_mock.assert_called_once_with(url, verify=True,
+                                          timeout=60)
+
     @mock.patch.object(requests, 'head', autospec=True)
     def _test_show(self, head_mock, mtime, mtime_date):
         head_mock.return_value.status_code = http_client.OK
@@ -206,7 +232,7 @@ class HttpImageServiceTestCase(base.TestCase):
         head_mock.assert_called_once_with(self.href, verify=True,
                                           timeout=60)
         self.assertEqual({'size': 100, 'updated_at': mtime_date,
-                          'properties': {}}, result)
+                          'properties': {}, 'no_cache': False}, result)
 
     def test_show_rfc_822(self):
         self._test_show(mtime='Tue, 15 Nov 2014 08:12:31 GMT',
@@ -220,6 +246,33 @@ class HttpImageServiceTestCase(base.TestCase):
         self._test_show(mtime='Tue Nov 15 08:12:31 2014',
                         mtime_date=datetime.datetime(2014, 11, 15, 8, 12, 31))
 
+    @mock.patch.object(requests, 'head', autospec=True)
+    def _test_show_with_cache(self, head_mock, cache_control, no_cache):
+        head_mock.return_value.status_code = http_client.OK
+        head_mock.return_value.headers = {
+            'Content-Length': 100,
+            'Last-Modified': 'Tue, 15 Nov 2014 08:12:31 GMT',
+            'Cache-Control': cache_control,
+        }
+        result = self.service.show(self.href)
+        head_mock.assert_called_once_with(self.href, verify=True,
+                                          timeout=60)
+        self.assertEqual({
+            'size': 100,
+            'updated_at': datetime.datetime(2014, 11, 15, 8, 12, 31),
+            'properties': {},
+            'no_cache': no_cache}, result)
+
+    def test_show_cache_allowed(self):
+        self._test_show_with_cache(
+            # Just because we cannot have nice things, "no-cache" actually
+            # means "cache, but always re-validate".
+            cache_control='no-cache, private', no_cache=False)
+
+    def test_show_cache_disabled(self):
+        self._test_show_with_cache(
+            cache_control='no-store', no_cache=True)
+
     @mock.patch.object(requests, 'head', autospec=True)
     def test_show_no_content_length(self, head_mock):
         head_mock.return_value.status_code = http_client.OK
@@ -426,122 +479,77 @@ class FileImageServiceTestCase(base.TestCase):
         self.assertEqual({'size': 42,
                           'updated_at': datetime.datetime(2015, 5, 8,
                                                           12, 25, 9, 164191),
-                          'properties': {}}, result)
+                          'properties': {},
+                          'no_cache': True}, result)
 
+    @mock.patch.object(shutil, 'copyfile', autospec=True)
     @mock.patch.object(os, 'link', autospec=True)
+    @mock.patch.object(os.path, 'realpath', lambda p: p)
     @mock.patch.object(os, 'remove', autospec=True)
-    @mock.patch.object(os, 'access', return_value=True, autospec=True)
-    @mock.patch.object(os, 'stat', autospec=True)
     @mock.patch.object(image_service.FileImageService, 'validate_href',
                        autospec=True)
-    def test_download_hard_link(self, _validate_mock, stat_mock, access_mock,
-                                remove_mock, link_mock):
+    def test_download_hard_link(self, _validate_mock, remove_mock, link_mock,
+                                copy_mock):
         _validate_mock.return_value = self.href_path
-        stat_mock.return_value.st_dev = 'dev1'
         file_mock = mock.Mock(spec=io.BytesIO)
         file_mock.name = 'file'
         self.service.download(self.href, file_mock)
         _validate_mock.assert_called_once_with(mock.ANY, self.href)
-        self.assertEqual(2, stat_mock.call_count)
-        access_mock.assert_called_once_with(self.href_path, os.R_OK | os.W_OK)
         remove_mock.assert_called_once_with('file')
         link_mock.assert_called_once_with(self.href_path, 'file')
+        copy_mock.assert_not_called()
 
-    @mock.patch.object(sendfile, 'sendfile', return_value=42, autospec=True)
-    @mock.patch.object(os.path, 'getsize', return_value=42, autospec=True)
-    @mock.patch.object(builtins, 'open', autospec=True)
-    @mock.patch.object(os, 'access', return_value=False, autospec=True)
-    @mock.patch.object(os, 'stat', autospec=True)
+    @mock.patch.object(shutil, 'copyfile', autospec=True)
+    @mock.patch.object(os, 'link', autospec=True)
+    @mock.patch.object(os, 'remove', autospec=True)
     @mock.patch.object(image_service.FileImageService, 'validate_href',
                        autospec=True)
-    def test_download_copy(self, _validate_mock, stat_mock, access_mock,
-                           open_mock, size_mock, copy_mock):
+    def test_download_copy(self, _validate_mock, remove_mock, link_mock,
+                           copy_mock):
         _validate_mock.return_value = self.href_path
-        stat_mock.return_value.st_dev = 'dev1'
+        link_mock.side_effect = PermissionError
         file_mock = mock.MagicMock(spec=io.BytesIO)
         file_mock.name = 'file'
-        input_mock = mock.MagicMock(spec=io.BytesIO)
-        open_mock.return_value = input_mock
         self.service.download(self.href, file_mock)
         _validate_mock.assert_called_once_with(mock.ANY, self.href)
-        self.assertEqual(2, stat_mock.call_count)
-        access_mock.assert_called_once_with(self.href_path, os.R_OK | os.W_OK)
-        copy_mock.assert_called_once_with(file_mock.fileno(),
-                                          input_mock.__enter__().fileno(),
-                                          0, 42)
+        link_mock.assert_called_once_with(self.href_path, 'file')
+        copy_mock.assert_called_once_with(self.href_path, 'file')
 
-    @mock.patch.object(sendfile, 'sendfile', autospec=True)
-    @mock.patch.object(os.path, 'getsize', return_value=42, autospec=True)
-    @mock.patch.object(builtins, 'open', autospec=True)
-    @mock.patch.object(os, 'access', return_value=False, autospec=True)
-    @mock.patch.object(os, 'stat', autospec=True)
+    @mock.patch.object(shutil, 'copyfile', autospec=True)
+    @mock.patch.object(os, 'link', autospec=True)
+    @mock.patch.object(os.path, 'realpath', autospec=True)
+    @mock.patch.object(os, 'remove', autospec=True)
     @mock.patch.object(image_service.FileImageService, 'validate_href',
                        autospec=True)
-    def test_download_copy_segmented(self, _validate_mock, stat_mock,
-                                     access_mock, open_mock, size_mock,
-                                     copy_mock):
-        # Fake a 3G + 1k image
-        chunk_size = image_service.SENDFILE_CHUNK_SIZE
-        fake_image_size = chunk_size * 3 + 1024
-        fake_chunk_seq = [chunk_size, chunk_size, chunk_size, 1024]
+    def test_download_symlink(self, _validate_mock, remove_mock,
+                              realpath_mock, link_mock, copy_mock):
         _validate_mock.return_value = self.href_path
-        stat_mock.return_value.st_dev = 'dev1'
+        realpath_mock.side_effect = lambda p: p + '.real'
         file_mock = mock.MagicMock(spec=io.BytesIO)
         file_mock.name = 'file'
-        input_mock = mock.MagicMock(spec=io.BytesIO)
-        open_mock.return_value = input_mock
-        size_mock.return_value = fake_image_size
-        copy_mock.side_effect = fake_chunk_seq
         self.service.download(self.href, file_mock)
         _validate_mock.assert_called_once_with(mock.ANY, self.href)
-        self.assertEqual(2, stat_mock.call_count)
-        access_mock.assert_called_once_with(self.href_path, os.R_OK | os.W_OK)
-        copy_calls = [mock.call(file_mock.fileno(),
-                                input_mock.__enter__().fileno(),
-                                chunk_size * i,
-                                fake_chunk_seq[i]) for i in range(4)]
-        copy_mock.assert_has_calls(copy_calls)
-        size_mock.assert_called_once_with(self.href_path)
+        realpath_mock.assert_called_once_with(self.href_path)
+        link_mock.assert_called_once_with(self.href_path + '.real', 'file')
+        copy_mock.assert_not_called()
 
-    @mock.patch.object(os, 'remove', side_effect=OSError, autospec=True)
-    @mock.patch.object(os, 'access', return_value=True, autospec=True)
-    @mock.patch.object(os, 'stat', autospec=True)
+    @mock.patch.object(shutil, 'copyfile', autospec=True)
+    @mock.patch.object(os, 'link', autospec=True)
+    @mock.patch.object(os, 'remove', autospec=True)
     @mock.patch.object(image_service.FileImageService, 'validate_href',
                        autospec=True)
-    def test_download_hard_link_fail(self, _validate_mock, stat_mock,
-                                     access_mock, remove_mock):
+    def test_download_copy_fail(self, _validate_mock, remove_mock, link_mock,
+                                copy_mock):
         _validate_mock.return_value = self.href_path
-        stat_mock.return_value.st_dev = 'dev1'
+        link_mock.side_effect = PermissionError
+        copy_mock.side_effect = PermissionError
         file_mock = mock.MagicMock(spec=io.BytesIO)
         file_mock.name = 'file'
         self.assertRaises(exception.ImageDownloadFailed,
                           self.service.download, self.href, file_mock)
         _validate_mock.assert_called_once_with(mock.ANY, self.href)
-        self.assertEqual(2, stat_mock.call_count)
-        access_mock.assert_called_once_with(self.href_path, os.R_OK | os.W_OK)
-
-    @mock.patch.object(sendfile, 'sendfile', side_effect=OSError,
-                       autospec=True)
-    @mock.patch.object(os.path, 'getsize', return_value=42, autospec=True)
-    @mock.patch.object(builtins, 'open', autospec=True)
-    @mock.patch.object(os, 'access', return_value=False, autospec=True)
-    @mock.patch.object(os, 'stat', autospec=True)
-    @mock.patch.object(image_service.FileImageService, 'validate_href',
-                       autospec=True)
-    def test_download_copy_fail(self, _validate_mock, stat_mock, access_mock,
-                                open_mock, size_mock, copy_mock):
-        _validate_mock.return_value = self.href_path
-        stat_mock.return_value.st_dev = 'dev1'
-        file_mock = mock.MagicMock(spec=io.BytesIO)
-        file_mock.name = 'file'
-        input_mock = mock.MagicMock(spec=io.BytesIO)
-        open_mock.return_value = input_mock
-        self.assertRaises(exception.ImageDownloadFailed,
-                          self.service.download, self.href, file_mock)
-        _validate_mock.assert_called_once_with(mock.ANY, self.href)
-        self.assertEqual(2, stat_mock.call_count)
-        access_mock.assert_called_once_with(self.href_path, os.R_OK | os.W_OK)
-        size_mock.assert_called_once_with(self.href_path)
+        link_mock.assert_called_once_with(self.href_path, 'file')
+        copy_mock.assert_called_once_with(self.href_path, 'file')
 
 
 class ServiceGetterTestCase(base.TestCase):
