@@ -21,14 +21,15 @@ import os
 import shutil
 from unittest import mock
 
-from ironic_lib import disk_utils
 from oslo_concurrency import processutils
 from oslo_config import cfg
 
 from ironic.common import exception
 from ironic.common.glance_service import service_utils as glance_utils
+from ironic.common import image_format_inspector
 from ironic.common import image_service
 from ironic.common import images
+from ironic.common import qemu_img
 from ironic.common import utils
 from ironic.tests import base
 
@@ -72,87 +73,185 @@ class IronicImagesTestCase(base.TestCase):
         image_to_raw_mock.assert_called_once_with(
             'image_href', 'path', 'path.part')
 
-    @mock.patch.object(disk_utils, 'qemu_img_info', autospec=True)
-    def test_image_to_raw_no_file_format(self, qemu_img_info_mock):
-        info = self.FakeImgInfo()
-        info.file_format = None
-        qemu_img_info_mock.return_value = info
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    def test_image_to_raw_not_permitted_format(self, detect_format_mock):
+        info = mock.MagicMock()
+        # In the case the image looks okay, but it is not in our permitted
+        # format list, we need to ensure we still fail appropriately.
+        info.safety_check.return_value = True
+        info.__str__.return_value = 'vhd'
+        detect_format_mock.return_value = info
 
         e = self.assertRaises(exception.ImageUnacceptable, images.image_to_raw,
                               'image_href', 'path', 'path_tmp')
-        qemu_img_info_mock.assert_called_once_with('path_tmp')
-        self.assertIn("'qemu-img info' parsing failed.", str(e))
+        info.safety_check.assert_called_once()
+        detect_format_mock.assert_called_once_with('path_tmp')
+        self.assertIn("The requested image is not valid for use.", str(e))
 
-    @mock.patch.object(disk_utils, 'qemu_img_info', autospec=True)
-    def test_image_to_raw_backing_file_present(self, qemu_img_info_mock):
-        info = self.FakeImgInfo()
-        info.file_format = 'raw'
-        info.backing_file = 'backing_file'
-        qemu_img_info_mock.return_value = info
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    def test_image_to_raw_fails_safety_check(self, detect_format_mock):
+        info = mock.MagicMock()
+        info.__str__.return_value = 'qcow2'
+        info.safety_check.return_value = False
+        detect_format_mock.return_value = info
 
         e = self.assertRaises(exception.ImageUnacceptable, images.image_to_raw,
                               'image_href', 'path', 'path_tmp')
-        qemu_img_info_mock.assert_called_once_with('path_tmp')
-        self.assertIn("fmt=raw backed by: backing_file", str(e))
+        info.safety_check.assert_called_once()
+        detect_format_mock.assert_called_once_with('path_tmp')
+        self.assertIn("The requested image is not valid for use.", str(e))
 
     @mock.patch.object(os, 'rename', autospec=True)
     @mock.patch.object(os, 'unlink', autospec=True)
-    @mock.patch.object(disk_utils, 'convert_image', autospec=True)
-    @mock.patch.object(disk_utils, 'qemu_img_info', autospec=True)
-    def test_image_to_raw(self, qemu_img_info_mock, convert_image_mock,
+    @mock.patch.object(qemu_img, 'convert_image', autospec=True)
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    def test_image_to_raw(self, detect_format_mock, convert_image_mock,
                           unlink_mock, rename_mock):
         CONF.set_override('force_raw_images', True)
-        info = self.FakeImgInfo()
-        info.file_format = 'fmt'
+        info = mock.MagicMock()
+        info.__str__.side_effect = iter(['qcow2', 'raw'])
         info.backing_file = None
-        qemu_img_info_mock.return_value = info
+        info.saftey_check.return_value = True
+        detect_format_mock.return_value = info
 
-        def convert_side_effect(source, dest, out_format):
+        def convert_side_effect(source, dest, out_format, source_format):
             info.file_format = 'raw'
         convert_image_mock.side_effect = convert_side_effect
 
         images.image_to_raw('image_href', 'path', 'path_tmp')
-
-        qemu_img_info_mock.assert_has_calls([mock.call('path_tmp'),
-                                             mock.call('path.converted')])
+        info.safety_check.assert_called_once()
+        self.assertEqual(2, info.__str__.call_count)
+        detect_format_mock.assert_has_calls([
+            mock.call('path_tmp'),
+            mock.call('path.converted')])
         convert_image_mock.assert_called_once_with('path_tmp',
-                                                   'path.converted', 'raw')
+                                                   'path.converted', 'raw',
+                                                   source_format='qcow2')
         unlink_mock.assert_called_once_with('path_tmp')
         rename_mock.assert_called_once_with('path.converted', 'path')
 
+    @mock.patch.object(os, 'rename', autospec=True)
     @mock.patch.object(os, 'unlink', autospec=True)
-    @mock.patch.object(disk_utils, 'convert_image', autospec=True)
-    @mock.patch.object(disk_utils, 'qemu_img_info', autospec=True)
-    def test_image_to_raw_not_raw_after_conversion(self, qemu_img_info_mock,
+    @mock.patch.object(qemu_img, 'convert_image', autospec=True)
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    def test_image_to_raw_safety_check_disabled(
+            self, detect_format_mock, convert_image_mock,
+            unlink_mock, rename_mock):
+        CONF.set_override('force_raw_images', True)
+        CONF.set_override('disable_deep_image_inspection', True,
+                          group='conductor')
+        info = mock.MagicMock()
+        info.__str__.side_effect = iter(['vmdk', 'raw'])
+        info.backing_file = None
+        info.saftey_check.return_value = None
+        detect_format_mock.return_value = info
+
+        def convert_side_effect(source, dest, out_format, source_format):
+            info.file_format = 'raw'
+        convert_image_mock.side_effect = convert_side_effect
+
+        images.image_to_raw('image_href', 'path', 'path_tmp')
+        info.safety_check.assert_not_called()
+        detect_format_mock.assert_has_calls([
+            mock.call('path')])
+        self.assertEqual(2, info.__str__.call_count)
+        convert_image_mock.assert_called_once_with('path_tmp',
+                                                   'path.converted', 'raw',
+                                                   source_format='vmdk')
+        unlink_mock.assert_called_once_with('path_tmp')
+        rename_mock.assert_called_once_with('path.converted', 'path')
+
+    @mock.patch.object(os, 'rename', autospec=True)
+    @mock.patch.object(os, 'unlink', autospec=True)
+    @mock.patch.object(qemu_img, 'convert_image', autospec=True)
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    def test_image_to_raw_safety_check_disabled_fails_to_convert(
+            self, detect_format_mock, convert_image_mock,
+            unlink_mock, rename_mock):
+        CONF.set_override('force_raw_images', True)
+        CONF.set_override('disable_deep_image_inspection', True,
+                          group='conductor')
+        info = mock.MagicMock()
+        info.__str__.return_value = 'vmdk'
+        info.backing_file = None
+        info.saftey_check.return_value = None
+        detect_format_mock.return_value = info
+
+        self.assertRaises(exception.ImageConvertFailed,
+                          images.image_to_raw,
+                          'image_href', 'path', 'path_tmp')
+        info.safety_check.assert_not_called()
+        self.assertEqual(2, info.__str__.call_count)
+        detect_format_mock.assert_has_calls([
+            mock.call('path')])
+        convert_image_mock.assert_called_once_with('path_tmp',
+                                                   'path.converted', 'raw',
+                                                   source_format='vmdk')
+        unlink_mock.assert_called_once_with('path_tmp')
+        rename_mock.assert_not_called()
+
+    @mock.patch.object(os, 'unlink', autospec=True)
+    @mock.patch.object(qemu_img, 'convert_image', autospec=True)
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    def test_image_to_raw_not_raw_after_conversion(self, detect_format_mock,
                                                    convert_image_mock,
                                                    unlink_mock):
         CONF.set_override('force_raw_images', True)
-        info = self.FakeImgInfo()
-        info.file_format = 'fmt'
-        info.backing_file = None
-        qemu_img_info_mock.return_value = info
+        info = mock.MagicMock()
+        info.__str__.return_value = 'qcow2'
+        detect_format_mock.return_value = info
 
         self.assertRaises(exception.ImageConvertFailed, images.image_to_raw,
                           'image_href', 'path', 'path_tmp')
-        qemu_img_info_mock.assert_has_calls([mock.call('path_tmp'),
-                                             mock.call('path.converted')])
         convert_image_mock.assert_called_once_with('path_tmp',
-                                                   'path.converted', 'raw')
+                                                   'path.converted', 'raw',
+                                                   source_format='qcow2')
         unlink_mock.assert_called_once_with('path_tmp')
+        info.safety_check.assert_called_once()
+        info.safety_check.assert_called_once()
+        self.assertEqual(2, info.__str__.call_count)
+        detect_format_mock.assert_has_calls([
+            mock.call('path_tmp'),
+            mock.call('path.converted')])
 
     @mock.patch.object(os, 'rename', autospec=True)
-    @mock.patch.object(disk_utils, 'qemu_img_info', autospec=True)
-    def test_image_to_raw_already_raw_format(self, qemu_img_info_mock,
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    def test_image_to_raw_already_raw_format(self, detect_format_mock,
                                              rename_mock):
-        info = self.FakeImgInfo()
-        info.file_format = 'raw'
-        info.backing_file = None
-        qemu_img_info_mock.return_value = info
+        info = mock.MagicMock()
+        info.__str__.return_value = 'raw'
+        detect_format_mock.return_value = info
 
         images.image_to_raw('image_href', 'path', 'path_tmp')
 
-        qemu_img_info_mock.assert_called_once_with('path_tmp')
         rename_mock.assert_called_once_with('path_tmp', 'path')
+        info.safety_check.assert_called_once()
+        info.safety_check.assert_called_once()
+        self.assertEqual(1, info.__str__.call_count)
+        detect_format_mock.assert_called_once_with('path_tmp')
+
+    @mock.patch.object(os, 'rename', autospec=True)
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    def test_image_to_raw_already_iso(self, detect_format_mock,
+                                      rename_mock):
+        info = mock.MagicMock()
+        info.__str__.return_value = 'iso'
+        detect_format_mock.return_value = info
+
+        images.image_to_raw('image_href', 'path', 'path_tmp')
+
+        rename_mock.assert_called_once_with('path_tmp', 'path')
+        info.safety_check.assert_called_once()
+        self.assertEqual(1, info.__str__.call_count)
+        detect_format_mock.assert_called_once_with('path_tmp')
 
     @mock.patch.object(image_service, 'get_image_service', autospec=True)
     def test_image_show_no_image_service(self, image_service_mock):
@@ -175,36 +274,39 @@ class IronicImagesTestCase(base.TestCase):
         show_mock.assert_called_once_with('context', 'image_href',
                                           'image_service')
 
-    @mock.patch.object(disk_utils, 'qemu_img_info', autospec=True)
-    def test_converted_size_estimate_default(self, qemu_img_info_mock):
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    def test_converted_size_estimate_default(self, image_info_mock):
         info = self.FakeImgInfo()
         info.disk_size = 2
         info.virtual_size = 10 ** 10
-        qemu_img_info_mock.return_value = info
+        image_info_mock.return_value = info
         size = images.converted_size('path', estimate=True)
-        qemu_img_info_mock.assert_called_once_with('path')
+        image_info_mock.assert_called_once_with('path')
         self.assertEqual(4, size)
 
-    @mock.patch.object(disk_utils, 'qemu_img_info', autospec=True)
-    def test_converted_size_estimate_custom(self, qemu_img_info_mock):
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    def test_converted_size_estimate_custom(self, image_info_mock):
         CONF.set_override('raw_image_growth_factor', 3)
         info = self.FakeImgInfo()
         info.disk_size = 2
         info.virtual_size = 10 ** 10
-        qemu_img_info_mock.return_value = info
+        image_info_mock.return_value = info
         size = images.converted_size('path', estimate=True)
-        qemu_img_info_mock.assert_called_once_with('path')
+        image_info_mock.assert_called_once_with('path')
         self.assertEqual(6, size)
 
-    @mock.patch.object(disk_utils, 'qemu_img_info', autospec=True)
-    def test_converted_size_estimate_raw_smaller(self, qemu_img_info_mock):
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    def test_converted_size_estimate_raw_smaller(self, image_info_mock):
         CONF.set_override('raw_image_growth_factor', 3)
         info = self.FakeImgInfo()
         info.disk_size = 2
         info.virtual_size = 5
-        qemu_img_info_mock.return_value = info
+        image_info_mock.return_value = info
         size = images.converted_size('path', estimate=True)
-        qemu_img_info_mock.assert_called_once_with('path')
+        image_info_mock.assert_called_once_with('path')
         self.assertEqual(5, size)
 
     @mock.patch.object(images, 'get_image_properties', autospec=True)
